@@ -1,5 +1,5 @@
 /*
- *   $Id: defaults.h,v 1.12 2005/07/05 08:10:28 psavola Exp $
+ *   $Id: defaults.h,v 1.15 2005/10/18 19:17:29 lutchann Exp $
  *
  *   Authors:
  *    Lars Fenneberg		<lf@elemental.net>	 
@@ -9,7 +9,7 @@
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lutchann@litech.org>.
+ *   may request it from <pekkas@netcore.fi>.
  *
  */
 
@@ -160,7 +160,7 @@ struct nd_opt_route_info_local     /* route information */
 /* Configurable values */
 
 #define DFLT_HomeAgentPreference	0
-#define DFLT_HomeAgentLifetime(iface)	DFLT_AdvDefaultLifetime(iface)
+#define DFLT_HomeAgentLifetime(iface)	((iface)->AdvDefaultLifetime)
 
 /* Other */
 
@@ -176,5 +176,17 @@ struct nd_opt_route_info_local     /* route information */
 
 /* #define MAX_RTR_SOLICITATIONS This MAY be ignored by MIPv6 */
 
+/* NEMO extensions, off by default */
+#define DFLT_AdvMobRtrSupportFlag      	0
+
+/* Flags */
+
+#ifndef ND_OPT_HAI_FLAG_SUPPORT_MR
+#if BYTE_ORDER == BIG_ENDIAN
+#define ND_OPT_HAI_FLAG_SUPPORT_MR	0x8000
+#else /* BYTE_ORDER == LITTLE_ENDIAN */
+#define ND_OPT_HAI_FLAG_SUPPORT_MR	0x0080
+#endif
+#endif
 
 #endif
