@@ -1,5 +1,5 @@
 /*
- *   $Id: interface.c,v 1.10 2005/03/29 15:15:20 psavola Exp $
+ *   $Id: interface.c,v 1.13 2005/12/30 16:12:23 psavola Exp $
  *
  *   Authors:
  *    Lars Fenneberg		<lf@elemental.net>	 
@@ -9,7 +9,7 @@
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lutchann@litech.org>.
+ *   may request it from <pekkas@netcore.fi>.
  *
  */
 
@@ -36,11 +36,12 @@ iface_init_defaults(struct Interface *iface)
 	iface->AdvHomeAgentFlag	  = DFLT_AdvHomeAgentFlag;
 	iface->HomeAgentPreference = DFLT_HomeAgentPreference;
 	iface->MinDelayBetweenRAs   = DFLT_MinDelayBetweenRAs;
+	iface->AdvMobRtrSupportFlag = DFLT_AdvMobRtrSupportFlag;
 
-	iface->MinRtrAdvInterval  = -1;
-	iface->AdvDefaultLifetime = -1;
+	iface->MinRtrAdvInterval = DFLT_MinRtrAdvInterval(iface);
+	iface->AdvDefaultLifetime = DFLT_AdvDefaultLifetime(iface);
 	iface->AdvDefaultPreference = DFLT_AdvDefaultPreference;
-	iface->HomeAgentLifetime  = 0;
+	iface->HomeAgentLifetime = DFLT_HomeAgentLifetime(iface);
 }
 
 void
@@ -92,9 +93,6 @@ check_iface(struct Interface *iface)
 		prefix = prefix->next;
 	}
 
-	if (iface->MinRtrAdvInterval == -1)
-		iface->MinRtrAdvInterval = DFLT_MinRtrAdvInterval(iface);
-
 	if ((iface->MinRtrAdvInterval < (MIPv6 ? MIN_MinRtrAdvInterval_MIPv6 : MIN_MinRtrAdvInterval)) || 
 		    (iface->MinRtrAdvInterval > MAX_MinRtrAdvInterval(iface)))
 	{
@@ -150,17 +148,6 @@ check_iface(struct Interface *iface)
 		res = -1;
 	}
 	
-	if (iface->AdvDefaultLifetime == -1)
-	{
-		iface->AdvDefaultLifetime = DFLT_AdvDefaultLifetime(iface);
-	}
-
-	/* Mobile IPv6 ext */
-	if (iface->HomeAgentLifetime == 0)
-	{
-		iface->HomeAgentLifetime = DFLT_HomeAgentLifetime(iface);
-	}
-
 	if ((iface->AdvDefaultLifetime != 0) &&
 	   ((iface->AdvDefaultLifetime > MAX_AdvDefaultLifetime) ||
 	    (iface->AdvDefaultLifetime < MIN_AdvDefaultLifetime(iface))))
@@ -193,6 +180,12 @@ check_iface(struct Interface *iface)
 			"AdvHomeAgentFlag for %s must be set with HomeAgentInfo", iface->Name);
 		res = -1;
 	}
+	if (iface->AdvMobRtrSupportFlag && !(iface->AdvHomeAgentInfo))
+	{
+		flog(LOG_ERR, 
+			"AdvHomeAgentInfo for %s must be set with AdvMobRtrSupportFlag", iface->Name);
+		res = -1;
+	}
 
 	/* XXX: need this? prefix = iface->AdvPrefixList; */
 
