@@ -1,5 +1,5 @@
 /*
- *   $Id: radvd.h,v 1.12 2005/02/15 08:32:06 psavola Exp $
+ *   $Id: radvd.h,v 1.17 2005/12/30 16:12:23 psavola Exp $
  *
  *   Authors:
  *    Pedro Roque		<roque@di.fc.ul.pt>
@@ -10,7 +10,7 @@
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lutchann@litech.org>.
+ *   may request it from <pekkas@netcore.fi>.
  *
  */
 
@@ -21,13 +21,14 @@
 #include <includes.h>
 #include <defaults.h>
 
-#define CONTACT_EMAIL	"Nathan Lutchansky <lutchann@litech.org>"
+#define CONTACT_EMAIL	"Pekka Savola <pekkas@netcore.fi>"
 
 /* for log.c */
 #define	L_NONE		0
 #define L_SYSLOG	1
 #define L_STDERR	2
-#define L_LOGFILE	3
+#define L_STDERR_SYSLOG	3
+#define L_LOGFILE	4
 
 #define LOG_TIME_FORMAT "%b %d %H:%M:%S"
 
@@ -47,7 +48,7 @@ struct Interface {
 	char			Name[IFNAMSIZ];	/* interface name */
 
 	struct in6_addr		if_addr;
-	int			if_index;
+	unsigned int		if_index;
 
 	uint8_t			if_hwaddr[HWADDR_MAX];
 	int			if_hwaddr_len;
@@ -61,11 +62,11 @@ struct Interface {
 	double			MinDelayBetweenRAs;
 	int			AdvManagedFlag;
 	int			AdvOtherConfigFlag;
-	int			AdvLinkMTU;
-	int			AdvReachableTime;
-	int			AdvRetransTimer;
-	int			AdvCurHopLimit;
-	int			AdvDefaultLifetime;
+	uint32_t		AdvLinkMTU;
+	uint32_t		AdvReachableTime;
+	uint32_t		AdvRetransTimer;
+	uint8_t			AdvCurHopLimit;
+	uint16_t		AdvDefaultLifetime;
 	int			AdvDefaultPreference;
 	int			AdvSourceLLAddress;
 	int			UnicastOnly;
@@ -74,9 +75,12 @@ struct Interface {
 	int			AdvIntervalOpt;
 	int			AdvHomeAgentInfo;
 	int			AdvHomeAgentFlag;
-	int16_t			HomeAgentPreference;
+	uint16_t		HomeAgentPreference;
 	uint16_t		HomeAgentLifetime;
 
+	/* NEMO extensions */
+	int			AdvMobRtrSupportFlag;
+
 	struct AdvPrefix	*AdvPrefixList;
 	struct AdvRoute		*AdvRouteList;
 	struct timer_lst	tm;
@@ -88,7 +92,7 @@ struct Interface {
 
 struct AdvPrefix {
 	struct in6_addr		Prefix;
-	int			PrefixLen;
+	uint8_t			PrefixLen;
 	
 	int			AdvOnLinkFlag;
 	int			AdvAutonomousFlag;
@@ -109,7 +113,7 @@ struct AdvPrefix {
 
 struct AdvRoute {
 	struct in6_addr		Prefix;
-	int			PrefixLen;
+	uint8_t			PrefixLen;
 	
 	int			AdvRoutePreference;
 	uint32_t		AdvRouteLifetime;
@@ -129,8 +133,8 @@ struct AdvInterval {
 struct HomeAgentInfo {
 	uint8_t			type;
 	uint8_t			length;
-	uint16_t		reserved;
-	int16_t			preference;
+	uint16_t		flags_reserved;
+	uint16_t		preference;
 	uint16_t		lifetime;
 };	
 
@@ -142,7 +146,7 @@ int yyparse(void);
 int yylex(void);
 
 /* radvd.c */
-int check_ip6_forwarding();
+int check_ip6_forwarding(void);
 
 /* timer.c */
 void set_timer(struct timer_lst *tm, double);
@@ -165,6 +169,10 @@ int setup_linklocal_addr(int, struct Interface *);
 int setup_allrouters_membership(int, struct Interface *);
 int check_allrouters_membership(int, struct Interface *);
 int get_v4addr(const char *, unsigned int *);
+int set_interface_linkmtu(const char *, uint32_t);
+int set_interface_curhlim(const char *, uint8_t);
+int set_interface_reachtime(const char *, uint32_t);
+int set_interface_retranstimer(const char *, uint32_t);
 
 /* interface.c */
 void iface_init_defaults(struct Interface *);
@@ -186,7 +194,7 @@ void process(int sock, struct Interface *, unsigned char *, int,
 int recv_rs_ra(int, unsigned char *, struct sockaddr_in6 *, struct in6_pktinfo **, int *);
 
 /* util.c */
-void mdelay(int);
+void mdelay(double);
 double rand_between(double, double);
 void print_addr(struct in6_addr *, char *);
 
