@@ -1,5 +1,5 @@
 /*
- *   $Id: util.c,v 1.4 2005/02/15 07:44:06 psavola Exp $
+ *   $Id: util.c,v 1.6 2005/12/30 15:13:11 psavola Exp $
  *
  *   Authors:
  *    Lars Fenneberg		<lf@elemental.net>	 
@@ -9,7 +9,7 @@
  *
  *   The license which is distributed with this software in the file COPYRIGHT
  *   applies to this software. If your distribution is missing this file, you
- *   may request it from <lutchann@litech.org>.
+ *   may request it from <pekkas@netcore.fi>.
  *
  */
 
@@ -18,7 +18,7 @@
 #include <radvd.h>
                
 void
-mdelay(int msecs)
+mdelay(double msecs)
 {
 	struct timeval tv;
                 
